"""Exceptions for Sinai."""


class MetricNotFound(Exception):
    """The search did not yield any result."""

    ...


class SourceError(Exception):
    """The Source did not return the required result."""

    ...
